;************************************************************************
;                                                                     	*
;    NOM:					Enregistreur de Temprature, liaison HF		*
;    Date:					Avril 2003                            		*
;    Version:				1.0                                   		*
;    Circuit:				Emetteur									*
;    Auteurs:				JEUFFRAULT/BERRADA 							*	
;                                                                     	*
;************************************************************************
;                                                                     	*
;    Fichier requis: P16F870.inc                                       	*
;                                                                     	*
;                                                                     	*
;                                                                     	*
;************************************************************************
;                                                                     	*
;    Notes:                                                           	*
;              Marche pour la premiere fois le 27/04/03                 *                                      *
;                                                                     	*
;                                                                     	*
;                                                                     	*
;************************************************************************


	LIST      p=16F870            ; Dfinition de processeur
	#include <p16F870.inc>        ; Dfinitions des constantes

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_OFF & _LVP_OFF & _BODEN_OFF & _XT_OSC

;	_CP_ON		Code protection ON : impossible de relire
;	_CP_OFF		Code protection OFF
;	_PWRTE_ON	Timer reset sur power on en service
;	_PWRTE_OFF	Timer reset hors-service

	ERRORLEVEL	-302	; Supprime le warning de BANK

;*********************************************************************
;                              ASSIGNATIONS                          *
;*********************************************************************

OPTIONVAL	EQU	0x80	; Valeur registre option
						; Rsistance pull-up OFF
INTCONMASK	EQU	0xC0	; Masque d'interruption	Interruption des peripheriques OK
INTPIE1MASK	EQU	0x01	; Enable	interruption sur debordement du timer1 (16 bits)

;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************
;DS
#DEFINE DQ		PORTA,0		; Ligne open drain

;Afficheur
#DEFINE RS			PORTA,1		; Selection du registre afficheur
#DEFINE E			PORTA,2		; Enable
#DEFINE	LCD_DATA	PORTB		; Afficheur LCD

;Emetteur
;#DEFINE OUTPUT		PORTC,6		; Sortie Vers module emission TX

;*********************************************************************
;                             MACROS                                 *
;*********************************************************************

BANK0	macro	
	bcf	STATUS,RP0
	endm

BANK1	macro	
	bsf	STATUS,RP0
	endm

Pin_low	macro
	BANK1
	bcf	DQ
	BANK0
	bcf	DQ
	endm

Pin_hz	macro
	BANK1
	bsf	DQ
	BANK0
	endm

VALIDE	macro	
	bsf	E
	nop
	bcf	E
	endm

;*********************************************************************
;                   DECLARATIONS DE VARIABLES                        *
;*********************************************************************

	CBLOCK 0x20   		; dbut de la zone variables

;valeurs de retour
	ds_return : 1		; Zone de 1 byte, valeur de retour du DS
	ds_bit:1
	Out_Byte :1

;compteurs	
	ds_cmpt:1
	cmpt1:1				
	cmpt2:1
	cmpt3 :1

;temperature
	Temp_Lsb : 1		; zone de 1 byte
	Temp_Msb : 1
;partie entiere
	Temp_diz : 1		
	Temp_cent : 1
	Temp_unit : 1
;partie decimale
	dec		:1
	un_sur_2	:1
	un_sur_4	:1
	un_sur_8	:1
	un_sur_16	:1
        ENDC			; Fin de la zone                        

;**********************************************************************
;                      DEMARRAGE SUR RESET                            *
;**********************************************************************

	org	0x000	 		; Adresse de dpart aprs reset
  	goto	Main		; Adresse 0: initialiser

;*********************************************************************
;                       INITIALISATIONS DU PIC                            *
;*********************************************************************

Init
	
	bcf	STATUS,RP1
	BANK0

	clrf	PORTA		; Sorties portA  0

	BANK1

	movlw	0x01		; PortA,0	en entre,  les autres en sorties		
	movwf	TRISA		
	
	movlw	0x06
	movwf	ADCON1		; entres sorties numriques
	
	clrf	TRISB		; portB en sortie
	clrf	TRISC		; portC ..

	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option
	
	BANK0
	
	clrf	PORTB		; sorties portB  0
	clrf	PORTC		; sorties portC  0

			; Effacer RAM
			
	movlw	0x20		; initialisation pointeur
	movwf	FSR			; pointeur d'adressage indirec
Init1
	clrf	INDF		; effacer ram
	incf	FSR,f		; pointer sur suivant
	btfss	FSR,7		; tester si fin zone atteinte (>=7F)
	goto	Init1		; non, boucler

	return

;*********************************************************************
;                      PROGRAMME PRINCIPAL                           *
;*********************************************************************

Main
		call	Init		;Initialisation PIC