;**********************************************************************
;    Ce programme est un programme didactique destin  monter        *
;    le fonctionnement des interruptions                              *
;                                                                     *
;**********************************************************************
;                                                                     *
;    NOM:       Interruption par bouton-poussoir sur RB0              *
;    Date:      13/02/2001                                            *
;    Version:   1.0                                                   *
;    Circuit:   Platine d'essais                                      *
;    Auteur:    Bigonoff                                              *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Fichier requis: P16F84.inc                                       *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:  Ce programme transforme un bouton-poussoir en            *
;            tlrupteur. Un pulse allume la LED, un autre            *
;            l'teint                                                 *
;                                                                     *
;                                                                     *
;**********************************************************************


	LIST      p=16F870            ; Dfinition de processeur
	#include <p16F870.inc>        ; Dfinitions de constantes

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_OFF & _LVP_OFF & _BODEN_OFF & _XT_OSC


;*********************************************************************
;                              ASSIGNATIONS                          *
;*********************************************************************

OPTIONVAL	EQU	0xC0	; Valeur registre option
                                ; Rsistance pull-up OFF
                                ; Interrupt flanc descendant RB0

INTERMASK	EQU	H'D0' ; Masque d'interruption
                                ; Interruptions sur RB0

INT_PIE1_MASK	EQU	0x01	; Interruption sur Timer 1

INT_T1CON_MASK	EQU	0x0A
;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************

#DEFINE	Bouton	PORTB , 0	; bouton poussoir
#DEFINE LED	PORTC , 2	; LED


;*********************************************************************
;                             MACRO                                  *
;*********************************************************************

BANK0	macro	
	bcf	STATUS,RP0	; passer en banque 0
	endm

BANK1	macro
	bsf	STATUS,RP0	; passer en banque1
	endm


;*********************************************************************
;                   DECLARATIONS DE VARIABLES                        *
;*********************************************************************

	CBLOCK 0x70   		; dbut de la zone variables
	w_temp :1		; Sauvegarde de W dans interruption
	status_temp : 1		; Sauvegarde de STATUS dans interrupt
	cmpt1 : 1		; compteur de boucles 1 dans tempo
	cmpt2 : 1		; compteur de boucles 2 dans tempo
	cmpt3 : 1	
       	ENDC			; Fin de la zone                        



;**********************************************************************
;                      DEMARRAGE SUR RESET                            *
;**********************************************************************

	org 0x000 		; Adresse de dpart aprs reset
  	goto    init		; Adresse 0: initialiser

;**********************************************************************
;                     ROUTINE INTERRUPTION                            *
;**********************************************************************

			;sauvegarder registres	
			;---------------------
	org 	0x004		; adresse d'interruption
	movwf   w_temp  	; sauver registre W
 	swapf	STATUS,w	; swap status avec rsultat dans w
	movwf	status_temp	; sauver status swapp

	btfsc	INTCON,INTF
	call	intrb0		; oui, traiter interrupt RB0
	btfsc	PIR1,TMR1IF
	call	intrtimer1
	bcf	PIR1,TMR1IF

			;restaurer registres
			;-------------------

	swapf	status_temp,w	; swap ancien status, rsultat dans w
	movwf   STATUS		; restaurer status
	swapf   w_temp,f	; Inversion L et H de l'ancien W
                       		; sans modifier Z
	swapf   w_temp,w  	; Rinversion de L et H dans W
				; W restaur sans modifier status
	retfie  		; return from interrupt

;**********************************************************************
;                     INTERRUPTION RB0/INT                            *
;**********************************************************************
;----------------------------------------------------------------------
;----------------------------------------------------------------------
intrb0
	BANK0		; car on ne sait pas sur quelle banque
	bsf		LED			

	movlw	.10
	call delay_100ms	; positionner flag tempo

	bcf	LED
	bcf	INTCON,INTF	; effacer flag INT/RB0
	clrf	TMR1H
	clrf	TMR1L
	return			; fin d'interruption RB0/INT

intrtimer1
	BANK0
	bsf	LED			

	movlw	.10
	call 	delay_100ms

	bcf	LED

	clrf	TMR1H
	clrf	TMR1L

	return                                                                         
;*********************************************************************
;                       INITIALISATIONS                              *
;*********************************************************************

init
	clrf	PORTB		; sorties portB  0
	clrf	PORTC
	BANK1			; passer banque1

	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option

			; configuer PORTS
			; ---------------
	bcf		LED		;  en sortie (TRISA)
	bsf		Bouton

	BANK0			; passer banque0

	movlw	INT_T1CON_MASK
	movwf	T1CON

	movlw	INTERMASK	; masque interruption
	movwf	INTCON		; charger interrupt control

	BANK1
	movlw	INT_PIE1_MASK
	movwf	PIE1
	BANK0
	clrf	PIR1

	clrf	TMR1L
	clrf	TMR1H

	bsf		T1CON,0		; Timer1 ON         	

	goto	start		; sauter programme principal

;*********************************************************************
;                      SOUS-ROUTINE DE TEMPORISATION                 *
;*********************************************************************
;---------------------------------------------------------------------
; Cette sous-routine introduit un retard
; Elle ne reoit aucun paramtre et n'en retourne aucun
;---------------------------------------------------------------------

delay_100ms
		movwf	cmpt3
delay_100ms_1
		movlw	.129
		movwf	cmpt2
		clrf	cmpt1
delay_100ms_2
		decfsz	cmpt1,f
		goto	delay_100ms_2
		nop
		nop
		nop
		nop
		nop
		decfsz	cmpt2,f
		goto	delay_100ms_2	
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop		; 20s
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		decfsz	cmpt3,f
		goto	delay_100ms_1
		return

;*********************************************************************
;                      PROGRAMME PRINCIPAL                           *
;*********************************************************************

start
	goto	start		; non, attendre qu'il soit mis
	END 			; directive fin de programme

