;**********************************************************************
;    Ce programme est un programme didactique destin  monter        *
;    le fonctionnement des interruptions                              *
;                                                                     *
;**********************************************************************
;                                                                     *
;    NOM:       Interruption par bouton-poussoir sur RB0              *
;    Date:      13/02/2001                                            *
;    Version:   1.0                                                   *
;    Circuit:   Platine d'essais                                      *
;    Auteur:    Bigonoff                                              *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Fichier requis: P16F84.inc                                       *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:  Ce programme transforme un bouton-poussoir en            *
;            tlrupteur. Un pulse allume la LED, un autre            *
;            l'teint                                                 *
;                                                                     *
;                                                                     *
;**********************************************************************


	LIST      p=16F870            ; Dfinition de processeur
	#include <p16F870.inc>        ; Dfinitions de constantes

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_OFF & _LVP_OFF & _BODEN_OFF & _XT_OSC


;*********************************************************************
;                              ASSIGNATIONS                          *
;*********************************************************************

OPTIONVAL	EQU	0x87	; Valeur registre option
                                ; Rsistance pull-up OFF
                                ; Interrupt flanc descendant RB0

INTERMASK	EQU	H'0090' ; Masque d'interruption
                                ; Interruptions sur RB0

;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************

#DEFINE	Bouton	PORTB , 0	; bouton poussoir
#DEFINE LED		PORTC , 0	; LED


;*********************************************************************
;                             MACRO                                  *
;*********************************************************************

BANK0	macro	
	bcf	STATUS,RP0	; passer en banque 0
	endm

BANK1	macro
	bsf	STATUS,RP0	; passer en banque1
	endm


;*********************************************************************
;                   DECLARATIONS DE VARIABLES                        *
;*********************************************************************

	CBLOCK 0x00C   		; dbut de la zone variables
	w_temp :1		; Sauvegarde de W dans interruption
	status_temp : 1		; Sauvegarde de STATUS dans interrupt
	cmpt1 : 1		; compteur de boucles 1 dans tempo
	cmpt2 : 1		; compteur de boucles 2 dans tempo
	cmpt3 : 1	
	flags : 1		; un octet pour 8 flags
				; rservons b0 pour le flag tempo
				; b1 : libre
				; b2 : libre
				; b3 : libre
				; b4 : libre
				; b5 : libre
				; b6 : libre
				; b7 : libre

       	ENDC			; Fin de la zone                        

#DEFINE	tempoF	flags , 0	; Dfinition du flag tempo

;**********************************************************************
;                      DEMARRAGE SUR RESET                            *
;**********************************************************************

	org 0x000 		; Adresse de dpart aprs reset
  	goto    init		; Adresse 0: initialiser

;**********************************************************************
;                     ROUTINE INTERRUPTION                            *
;**********************************************************************

			;sauvegarder registres	
			;---------------------
	org 0x004		; adresse d'interruption
	movwf   w_temp  	; sauver registre W
	swapf	STATUS,w	; swap status avec rsultat dans w
	movwf	status_temp	; sauver status swapp


	call	intrb0		; oui, traiter interrupt RB0

			;restaurer registres
			;-------------------

	swapf	status_temp,w	; swap ancien status, rsultat dans w
	movwf   STATUS		; restaurer status
	swapf   w_temp,f	; Inversion L et H de l'ancien W
                       		; sans modifier Z
	swapf   w_temp,w  	; Rinversion de L et H dans W
				; W restaur sans modifier status
	retfie  		; return from interrupt

;**********************************************************************
;                     INTERRUPTION RB0/INT                            *
;**********************************************************************
;----------------------------------------------------------------------
; inverse le niveau de RA2  chaque passage
; interdit toute nouvelle interruption
; valide le flag tempo
;----------------------------------------------------------------------
intrb0
	movlw	B'00000001'	; bit positionn = bit invers
	BANK0			; car on ne sait pas sur quelle banque
	xorwf	PORTC , f	; inverser RA2	
	bcf	INTCON , INTF	; effacer flag INT/RB0
	bcf	INTCON , INTE	; interdire autre inter. RB0
	bsf	tempoF		; positionner flag tempo
	return			; fin d'interruption RB0/INT

;*********************************************************************
;                       INITIALISATIONS                              *
;*********************************************************************

init
	clrf	PORTA		; Sorties portA  0
	clrf	PORTB		; sorties portB  0
	clrf	PORTC
	BANK1			; passer banque1

	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option

			; Effacer RAM
			; ------------
	movlw	0x20		; initialisation pointeur
	movwf	FSR		; pointeur d'adressage indirect
init1
	clrf	INDF		; effacer ram
	incf	FSR,f		; pointer sur suivant
	btfss	FSR,7		; tester si fin zone atteinte (>=40)
	goto	init1		; non, boucler


			; configuer PORTS
			; ---------------
	bcf		LED		; RA2 en sortie (TRISA)
	bsf		Bouton

	BANK0			; passer banque0
	movlw	INTERMASK	; masque interruption
	movwf	INTCON		; charger interrupt control
	goto	start		; sauter programme principal

;*********************************************************************
;                      SOUS-ROUTINE DE TEMPORISATION                 *
;*********************************************************************
;---------------------------------------------------------------------
; Cette sous-routine introduit un retard
; Elle ne reoit aucun paramtre et n'en retourne aucun
;---------------------------------------------------------------------

tempo
		movlw	.5
		movwf	cmpt3
delay_100ms_1
		movlw	.129
		movwf	cmpt2
		clrf	cmpt1
delay_100ms_2
		decfsz	cmpt1,f
		goto	delay_100ms_2
		nop
		nop
		nop
		nop
		nop
		decfsz	cmpt2,f
		goto	delay_100ms_2	
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop		; 20s
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		decfsz	cmpt3,f
		goto	delay_100ms_1
		return

;*********************************************************************
;                      PROGRAMME PRINCIPAL                           *
;*********************************************************************

start
	btfss	tempoF		; tester si tempo flag mis
	goto	start		; non, attendre qu'il soit mis
	call 	tempo		; oui, excuter tempo
	bcf	tempoF		; effacer flag tempo
	bcf	INTCON , INTF	; effacer flag INT
	bsf	INTCON , INTE	; remettre interrupts INT en service
	goto start		; boucler
	END 			; directive fin de programme

