;************************************************************************
;                                                                     	*
;    NOM:				Enregistreur de Temprature, liaison HF			*
;    Date:				Avril 2003                            			*
;    Version:			1.0                                   			*
;    Circuit:			Recepteur										*
;    Auteurs:			JEUFFRAULT/BERRADA 								*	
;                                                                     	*
;************************************************************************
;                                                                     	*
;    Fichier requis: P16F870.inc                                       	*
;                                                                     	*
;                                                                     	*
;                                                                     	*
;************************************************************************
;                                                                     	*
;    Notes:                                                           	*
;            						Quartz 4.000 Mhz                    *
;                                                                     	*
;                                                                     	*
;                                                                     	*
;************************************************************************


	LIST      p=16F870           ; Dfinition de processeur
	#include <p16F870.inc>       ; Dfinitions des constantes

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_OFF & _LVP_OFF & _BODEN_OFF & _CPD_OFF & _XT_OSC

;	_CP_ON		Code protection ON : impossible de relire
;	_CP_OFF		Code protection OFF
;	_PWRTE_ON	Timer reset sur power on en service
;	_PWRTE_OFF	Timer reset hors-service

	ERRORLEVEL	-302			; Supprime le warning de BANK

;************************************************************************
;	                             DEFINE                                 *
;************************************************************************

#DEFINE	BOUTON		PORTB,0
#DEFINE	LED			PORTC,3

;************************************************************************
;	                             MACROS                              	*
;************************************************************************

BANK0	macro	
	bcf		STATUS,RP0
	bcf		STATUS,RP1
	endm

BANK1	macro	
	bsf		STATUS,RP0
	bcf		STATUS,RP1
	endm

BANK2	macro	
	bcf		STATUS,RP0
	bsf		STATUS,RP1
	endm

BANK3	macro	
	bsf		STATUS,RP0
	bsf		STATUS,RP1
	endm

;************************************************************************
;	                   DECLARATIONS DE VARIABLES                        *
;************************************************************************

	CBLOCK 0x20   				; dbut de la zone variables

;compteurs	
	cmpt1		:1				
	cmpt2		:1
	cmpt3 		:1

	cmpt_flash	:1
        ENDC					; Fin de la zone   
        
 	CBLOCK 0x70
 	high_adresse:1
	low_adresse	:1	
		ENDC
 	                            

;************************************************************************
;	                      DEMARRAGE SUR RESET                           *
;************************************************************************

	org		0x000	 			; Adresse de dpart aprs reset
  	goto	Main				; Adresse 0: initialiser


;************************************************************************
;	                     ROUTINE INTERRUPTION                           *
;************************************************************************

	org 	0x004					; adresse d'interruption

	bsf		LED

	BANK1
	bsf		TXSTA,TXEN	
	BANK0
	
	call	aff_message
		
	movlw	.24
	movwf	cmpt_flash
	
	bcf		LED
	
	movlw	.50
	call	delay_100ms		
	
	retfie


aff_message
	call	read_flash
	decfsz	cmpt_flash,f
	goto	continue
	return
	
continue	
	incf	low_adresse,f	
ready
	btfss	PIR1,TXIF
	goto	ready
	goto	aff_message
	
	
read_flash
	BANK2
	movf	high_adresse,W
	movwf	EEADRH
	movf	low_adresse,W
	movwf	EEADR
	BANK3
	bsf		EECON1,EEPGD
	bsf		EECON1,RD
	BANK2
	nop
	movf	EEDATA,W
	BANK0
	movwf	TXREG
	return



;************************************************************************
;	                       INITIALISATIONS DU PIC                       *
;************************************************************************

Init

	BANK0

	clrf	PORTA				; Sorties portA  0
	clrf	PORTB				; sorties portB  0
	clrf	PORTC				; sorties portC  0

	BANK1

	movlw	0x01				; PORTB,0	en entre,  les autres en sorties		
	movwf	TRISB		
	
	movlw	0x06
	movwf	ADCON1				; entres sorties numriques
	
	clrf	TRISA				; portA en sortie

	movlw	0x80
	movwf	TRISC				; portC en sortie sauf RC6 et RC7 pour l'USART ..

	movlw	0xC0		 		; interupt front montant RB0
	movwf	OPTION_REG			

	movlw	.12					; liaison a 19200 bauds
	movwf	SPBRG
	
	movlw	0x06				
	movwf	TXSTA
	
	movlw	0x20
	movwf	PIE1	
	
	BANK0
	
	movlw	0x90
	movwf	RCSTA	

	movlw	0xC0				; masque interruption
	movwf	INTCON				; charger interrupt periph

	
			; Effacer RAM
			
	movlw	0x20				; initialisation pointeur
	movwf	FSR					; pointeur d'adressage indirec
Init1
	clrf	INDF				; effacer ram
	incf	FSR,f				; pointer sur suivant
	btfss	FSR,7				; tester si fin zone atteinte (>=7F)
	goto	Init1				; non, boucler



	movlw	0x07
	movwf	high_adresse
	movlw	0xE0	
	movwf	low_adresse
	movlw	.24
	movwf	cmpt_flash

	return



;************************************************************************
;	                      PROGRAMME PRINCIPAL                        	*
;************************************************************************

Main
		call	Init			; Initialisation PIC
Main1
		goto	Main1


;************************************************************************
;                  	BOUCLES 	DE	TEMPORISATION 						*
;										10s	,1ms	,100ms     		*
;************************************************************************

;************************************************************************
;                  				10sec*W + 4 s           				*
;************************************************************************

delay_10u
		movwf	cmpt1	
delay_10u_1
		NOP	
		NOP
		NOP
		NOP	
		NOP
		NOP
		NOP
		decfsz	cmpt1,F
		goto	delay_10u_1
		return

;************************************************************************
;								1msec*W  +4s							*
;************************************************************************

delay_1ms
		movwf	cmpt2
delay_1ms_1
		clrf	cmpt1
delay
		decfsz	cmpt1,f
		goto	delay
	
		movlw	.76
		movwf	cmpt1
delay2	
		decfsz	cmpt1,f
		goto	delay2

		decfsz	cmpt2,f
		goto	delay_1ms_1
	
		return


;************************************************************************
;								100msec*W  + 4 s             			*
;************************************************************************

delay_100ms
		movwf	cmpt3
delay_100ms_1
		movlw	.129
		movwf	cmpt2
		clrf	cmpt1
delay_100ms_2
		decfsz	cmpt1,f
		goto	delay_100ms_2
		nop
		nop
		nop
		nop
		nop
		decfsz	cmpt2,f
		goto	delay_100ms_2	
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop		; 20s
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		decfsz	cmpt3,f
		goto	delay_100ms_1
		return
		
		

;************************************************************************
;	             INITIALISATION D'UNE ZONE MEMOIRE FLASH				*
;************************************************************************

	Org	0x07E0
	DA	'L','a',' ','T','e','m','p','e','r','a','t','u','r','e',' ','e','s','t',' ','d','e',':',0x0D,0x0A

		
		
		
	END 						; directive fin de programme
