;************************************************************************
;                                                                     	*
;    NOM:					Enregistreur de Temprature, liaison HF		*
;    Date:					Avril 2003                            		*
;    Version:				1.0                                   		*
;    Circuit:				Emetteur									*
;    Auteurs:				JEUFFRAULT/BERRADA 							*	
;                                                                     	*
;************************************************************************
;                                                                     	*
;    Fichier requis: P16F870.inc                                       	*
;                                                                     	*
;                                                                     	*
;                                                                     	*
;************************************************************************
;                                                                     	*
;    Notes:                                                           	*
;              Marche pour la premiere fois le 27/04/03                 *                                      *
;                                                                     	*
;                                                                     	*
;                                                                     	*
;************************************************************************


	LIST      p=16F870            ; Dfinition de processeur
	#include <p16F870.inc>        ; Dfinitions des constantes

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_OFF & _LVP_OFF & _BODEN_OFF & _XT_OSC

;	_CP_ON		Code protection ON : impossible de relire
;	_CP_OFF		Code protection OFF
;	_PWRTE_ON	Timer reset sur power on en service
;	_PWRTE_OFF	Timer reset hors-service

	ERRORLEVEL	-302	; Supprime le warning de BANK

;*********************************************************************
;                              ASSIGNATIONS                          *
;*********************************************************************

OPTIONVAL	EQU	0x80	; Valeur registre option
						; Rsistance pull-up OFF
INTCONMASK	EQU	0xC0	; Masque d'interruption	Interruption des peripheriques OK
INTPIE1MASK	EQU	0x01	; Enable	interruption sur debordement du timer1 (16 bits)
CONFIG_T1	EQU	0x31	; Configuration Timer1 : prescaler *8, clock interne, timer enable

;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************
;DS
#DEFINE DQ			PORTA,0		; Ligne open drain

;Afficheur
#DEFINE RS			PORTA,1		; Selection du registre afficheur
#DEFINE E			PORTA,2		; Enable
#DEFINE	LCD_DATA	PORTB		; Afficheur LCD

TIMEBASE	EQU		.114		; base de temps 114*65536*8 = 59.77 Secondes

;Emetteur
;#DEFINE OUTPUT		PORTC,6		; Sortie Vers module emission TX

;*********************************************************************
;                             MACROS                                 *
;*********************************************************************

BANK0	macro	
	bcf		STATUS,RP0
	endm

BANK1	macro	
	bsf		STATUS,RP0
	endm

Pin_low	macro
	BANK1
	bcf		DQ
	BANK0
	bcf		DQ
	endm

Pin_hz	macro
	BANK1
	bsf		DQ
	BANK0
	endm

VALIDE	macro	
	bsf		E
	nop
	bcf		E
	endm

;*********************************************************************
;                   DECLARATIONS DE VARIABLES                        *
;*********************************************************************

	CBLOCK		0x20	; dbut de la zone variables

;valeurs de retour
	ds_return	:1		; Zone de 1 byte, valeur de retour du DS
	ds_bit		:1
	Out_Byte	:1

;compteurs	
	ds_cmpt		:1
	cmpt1		:1				
	cmpt2		:1
	cmpt3		:1

;temperature
	Temp_Lsb	:1		; zone de 1 byte
	Temp_Msb	:1
;partie entiere
	Temp_diz	:1		
	Temp_cent	:1
	Temp_unit 	:1
;partie decimale
	dec			:1
	un_sur_2	:1
	un_sur_4	:1
	un_sur_8	:1
	un_sur_16	:1

	w_temp		:1
	status_temp	:1
	cmpt_timer	:1
        ENDC			; Fin de la zone                        

;**********************************************************************
;                      DEMARRAGE SUR RESET                            *
;**********************************************************************

	org		0x000	 	; Adresse de dpart aprs reset
  	goto	Init		; Adresse 0: initialiser


;**********************************************************************
;                     ROUTINE INTERRUPTION                            *
;**********************************************************************

			;sauvegarder registres	
			;---------------------
	org		0x004			; adresse d'interruption
	movwf   w_temp  		; sauver registre W
	swapf	STATUS,w		; swap status avec rsultat dans w
	movwf	status_temp		; sauver status swapp

			; switch vers diffrentes interrupts
			;----------------------------------------
	
	btfsc	PIE1,TMR1IE		; tester si interrupt timer1 autorise
	btfss	PIR1,TMR1IF		; oui, tester si interrupt timer1 en cours
	goto 	intsw1			; non test suivant
	call	inttimer		; oui, traiter interrupt timer
	bcf		PIR1,TMR1IF		; effacer flag interrupt timer
	goto	restore_reg		; et fin d'interruption

intsw1
;	btfsc	INTCON,INTE		; tester si interrupt RB0 autorise
;	btfss	INTCON,INTF		; oui, tester si interrupt RB0 en cours
;	goto 	intsw2			; non sauter au test suivant
;	call	intrb0			; oui, traiter interrupt RB0
;	bcf		INTCON,INTF		; effacer flag interupt RB0
;	goto	restorereg		; et fin d'interruption
							; SUPPRIMER CETTE LIGNE POUR
							; TRAITER PLUSIEURS INTERRUPT
							; EN 1 SEULE FOIS


			;restaurer registres
			;----------------------------------------
restore_reg
	swapf	status_temp,w	; swap ancien status, rsultat dans w
	movwf   STATUS			; restaurer status
	swapf   w_temp,f		; Inversion L et H de l'ancien W, sans modifier Z
	swapf   w_temp,w  		; Rinversion de L et H dans W, W restaur sans modifier status
	retfie  				; return from interrupt


;*********************************************************************
;                       INITIALISATIONS DU PIC                       *
;*********************************************************************

Init
	
	bcf		STATUS,RP1
	BANK0

	clrf	PORTA		; Sorties portA  0

	BANK1

	movlw	0x01		; PortA,0	en entre,  les autres en sorties		
	movwf	TRISA		
	
	movlw	0x06
	movwf	ADCON1		; entres sorties numriques
	
	clrf	TRISB		; portB en sortie
	clrf	TRISC		; portC ..

	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option

	movlw	INTCONMASK	; charger masque INT
	movwf	INTCON		; initialiser registre INT

	movlw	INTPIE1MASK	; charger masque INT peripheriques
	movwf	PIE1		; initialiser registre INT per
	
	BANK0
	
	clrf	PORTB		; sorties portB  0
	clrf	PORTC		; sorties portC  0

			; Effacer RAM
			
	movlw	0x20		; initialisation pointeur
	movwf	FSR			; pointeur d'adressage indirec
Init1
	clrf	INDF		; effacer ram
	incf	FSR,f		; pointer sur suivant
	btfss	FSR,7		; tester si fin zone atteinte (>=7F)
	goto	Init1		; non, boucler

	call	LCD_Init	; Initialisation LCD 8 bits

	movlw	TIMEBASE
	movwf	cmpt_timer

	movlw	CONFIG_T1
	movwf	T1CON

	goto	Main


;**********************************************************************
;                     INTERRUPTION TIMER 0                            *
;**********************************************************************
inttimer
		decfsz	cmpt_timer,f
		return				; Si cmpt_timer n'est pas nul on ne fait et on s'endort

		bsf		PORTC,0	

		movlw	TIMEBASE
		movwf	cmpt_timer		

		BANK0
		call	InitDS18B20	;Initialisation DS18B20
		movf	ds_return,F
		btfsc	STATUS,Z
		goto	ERREUR_DS	;Erreur

		call	ReadTemp	;Lecture de la temperature
		movf	ds_return,F
		btfsc	STATUS,Z
		goto	ERREUR_DS	;Erreur

		movlw	'T'
		call	LCD_Write_car

		movlw	'='
		call	LCD_Write_car

		call	Affich_Temp	;Affichage de la temprature sur le LCD
			
; Laurent est un BOSS

		bcf		RS			;mode commande
		movlw	0xC0
		movwf	LCD_DATA
		VALIDE				; saut de ligne
		movlw	.2
		call	delay_1ms

		movlw	'L'
		call	LCD_Write_car

		movlw	'o'
		call	LCD_Write_car

		movlw	'l'
		call	LCD_Write_car

		movlw	'o'
		call	LCD_Write_car

		movlw	' '
		call	LCD_Write_car

		movlw	'e'
		call	LCD_Write_car

		movlw	's'
		call	LCD_Write_car

		movlw	't'
		call	LCD_Write_car

		movlw	' '
		call	LCD_Write_car

		movlw	'u'
		call	LCD_Write_car

		movlw	'n'
		call	LCD_Write_car

		movlw	' '
		call	LCD_Write_car

		movlw	'B'
		call	LCD_Write_car

		movlw	'O'
		call	LCD_Write_car

		movlw	'S'
		call	LCD_Write_car

		movlw	'S'
		call	LCD_Write_car

		bcf		RS		;mode commande
		movlw	0x02
		movwf	LCD_DATA
		VALIDE			; retour en dbut de ligne
		movlw	.2
		call	delay_1ms

		bcf		PORTC,0

		return			; fin d'interruption timer
			
;*********************************************************************
;                      PROGRAMME PRINCIPAL                           *
;*********************************************************************

Main
		sleep
		nop
		goto	Main

;*********************************************************************
;                      Initialisation DS18B20                        *
;*********************************************************************

InitDS18B20
		call	resetPresence
		movf	ds_return,F
		btfsc	STATUS,Z
		return	; resetPresence failed
		
		movlw	0xCC		; Skip ROM
		call	WriteTempByte
		movlw	0x4E		; Write Scratchpad
		call	WriteTempByte
		movlw	0x00		; Write TH
		call	WriteTempByte
		movlw	0x00		; Write TL
		call	WriteTempByte
		movlw	0x7F		; Write config du DS resolution de 12 Bits
		call	WriteTempByte

		movlw	.1
		movwf	ds_return
		
		return

;*********************************************************************
;                      Reset Presence	                             *
;*********************************************************************

resetPresence
		Pin_hz
		movlw	.0
		movwf	ds_return

		Pin_low
		movlw	.50
		call	delay_10u	; attendre 500 s

		Pin_hz				; Haute impdance
		movlw	.6
		call	delay_10u	; attendre 60 s
		btfsc	DQ
		return				; Erreur si DQ=1

		movlw	.6
		call	delay_10u	; attendre 60 s
;
;		btfsc	DQ
;		return

		movlw	.36
		call	delay_10u	; attendre 360 s
	
		movlw	.1
		movwf	ds_return

		return

;*********************************************************************
;                      ReadTemp                                      *
;*********************************************************************

ReadTemp
		call	resetPresence
		movf	ds_return,F
		btfsc	STATUS,Z
		return

		movlw	0xCC		; Skip ROM
		call	WriteTempByte

		movlw	0x44		; temp conversion
		call	WriteTempByte

Ds_loop_read
		movlw	.100
		call	delay_1ms
		call	ReadTempBit
		btfss	ds_bit,0
		goto	Ds_loop_read	; Boucle tant que processus de lecture non termin


		call	resetPresence
		movf	ds_return,F
		btfsc	STATUS,Z
		return					; resetPresence failed

		movlw	0xCC			; Skip ROM
		call	WriteTempByte

		movlw	0xBE			; Read Scratchpad
		call	WriteTempByte
		
		call	ReadTempByte
		movf	ds_return,W
		movwf	Temp_Lsb		; Ecriture du poids faible

		call	ReadTempByte
		movf	ds_return,W
		movwf	Temp_Msb		; Ecriture du poids fort

		call	resetPresence
		movf	ds_return,F
		btfsc	STATUS,Z
		return					; Erreur

		return

;*********************************************************************
;                     ReadTempByte                                   *
;*********************************************************************

ReadTempByte
		movlw	.8
		movwf	ds_cmpt
DS_boucle_rot
		call	ReadTempBit
		rrf		ds_bit,F
		rrf		ds_return,F
		decfsz	ds_cmpt,F
		goto	DS_boucle_rot
		
		return

;*********************************************************************
;                      ReadTempBit                                   *
;*********************************************************************

ReadTempBit
		Pin_low
		nop
		nop
		Pin_hz
		nop
		nop
		nop
		nop
		nop
		nop	; 8s en plus
		
		movf	PORTA,W
		movwf	ds_bit

		movlw	.6
		call	delay_10u
		
		return

;*********************************************************************
;                     WriteTempByte                                  *
;*********************************************************************

WriteTempByte
		movwf	Out_Byte
		movlw	.8
		movwf	ds_cmpt
WriteTempByte_1
		rrf		Out_Byte,F
		btfss	STATUS,C
		goto	Write_0
		goto	Write_1
WriteTempByte_2
		decfsz	ds_cmpt,F
		goto	WriteTempByte_1
		return

Write_0
		Pin_low
		movlw	.6
		call	delay_10u
		Pin_hz
		goto	WriteTempByte_2

Write_1
		Pin_low
		nop
		Pin_hz
		movlw	.6
		call	delay_10u
		goto	WriteTempByte_2

; * * * * * * * * * * * * * * * * * * * * ***
; LCD_Init		Initialisation du module LCD*
; * * * * * * * * * * * * * * * * * * * * ***

LCD_Init
	bcf		RS				; On met a 0 RS pour les commandes
	bcf		E

	movlw	.15
	call   delay_1ms  	; tempo 15 ms

	movlw	0x38
	movwf	LCD_DATA
	VALIDE				;	2 lignes 8 bits

	movlw	.5
	call	delay_1ms

	movlw	0x38
	movwf	LCD_DATA
	VALIDE				;	2 lignes 8 bits

	movlw	.10
	call	delay_10u

	movlw	0x38
	movwf	LCD_DATA
	VALIDE				;	2 lignes 8 bits

	movlw	.10
	call	delay_10u

	movlw	0x38
	movwf	LCD_DATA
	VALIDE				;	2 lignes 8 bits

	movlw	.10
	call	delay_10u

	movlw	0x0C
	movwf	LCD_DATA
	VALIDE				;Affichage ON [D=1(Affon), C=0(Cursr invisible)

	movlw	.10
	call	delay_10u

	movlw	0x01
	movwf	LCD_DATA
	VALIDE				; Clear display

	movlw	.2
	call   delay_1ms  	; tempo 2 ms

	movlw	0x06
	movwf	LCD_DATA
	VALIDE				; curseur se deplace tout seul

	movlw	.10
	call	delay_10u

	return
	
;******************************************************
;*  LCD_Write    On prend le caractres  partir de W *
;******************************************************

LCD_Write_ASCII
	addlw	'0'
LCD_Write_car
	bsf		RS				; On met a 1 RS pour les caractres
	movwf	LCD_DATA
	VALIDE

	movlw	.10
	call	delay_10u
	
	return

;*********************************************************************
;                      Affich_Temp				                     *
;*********************************************************************

Affich_Temp
	btfsc	Temp_Msb,3
	goto	Temp_neg
	
	movlw	'+'				; Temperature negative
	call	LCD_Write_car
	
	goto	Temp_pos
	
Temp_neg					; Temperature positive
	movlw	'-'
	call	LCD_Write_car
		
	comf	Temp_Msb,f		; Complement a 2 des 2 variables
	incf	Temp_Msb,f
	comf	Temp_Lsb,f
	incf	Temp_Lsb,f
		
Temp_pos

	call	Calcul_int_part	; Calcul de la partie entiere de la temperature
	
	movf	Temp_cent,W		; Centaine
	call	LCD_Write_ASCII
	
	movf	Temp_diz,W		; Dizaine
	call	LCD_Write_ASCII
	
	movf	Temp_unit,W		; Unit
	call	LCD_Write_ASCII

	movlw	'.'
	call	LCD_Write_car

	call	Calcul_dec_part	; Calcul de la partie dcimale

	movf	un_sur_2,w		; 2^-1
	call	LCD_Write_ASCII

	movf	un_sur_4,w		; 2^-2
	call	LCD_Write_ASCII

	movf	un_sur_8,w		; 2^-3
	call	LCD_Write_ASCII

	movf	un_sur_16,w		; 2^-4
	call	LCD_Write_ASCII

	movlw	0xDF		; caractre degr ""
	call	LCD_Write_car

	movlw	'C'			; Celsius
	call	LCD_Write_car
	
	return


;*********************************************************************
;                      Calcul_int_part				                 *
;*********************************************************************

Calcul_int_part
	clrf	dec
	
	rrf		Temp_Msb,f
	rrf		Temp_Lsb,f
	rrf		dec,f
	
	rrf		Temp_Msb,f
	rrf		Temp_Lsb,f
	rrf		dec,f
	
	rrf		Temp_Msb,f
	rrf		Temp_Lsb,f
	rrf		dec,f
	
	rrf		Temp_Msb,f
	rrf		Temp_Lsb,f
	rrf		dec,f		; 4 decalages pour obtenir la partie entire de la temprature dans Temp_Lsb

	swapf	dec,f		; La partie decimale est maintenant dans dec (poids faible)
	
	movf	Temp_Lsb,W
	
	clrf	Temp_cent
	clrf	Temp_diz
	clrf	Temp_unit
		
						; Dbut de la procdure de calcul des cantaines/dizaines/units
centaine			
	incf	Temp_cent,f
	
	addlw	-.100		;W-100->W
	btfsc	STATUS,Z
	goto	final
	btfsc	STATUS,C	
	goto	centaine
	
	addlw	.100
	decf	Temp_cent,f
	
dizaine
	incf	Temp_diz,f
	
	addlw	-.10		;W-10->W
	btfsc	STATUS,Z
	goto	final
	btfsc	STATUS,C
	goto	dizaine
	
	addlw	.10
	decf	Temp_diz,f

unite
	incf	Temp_unit,f
	
	addlw	-.1		;W-1->W
	btfsc	STATUS,Z
	goto	final
	btfsc	STATUS,C
	goto	unite
	
	addlw	.1
	decf	Temp_unit,f

final
	return


;*********************************************************************
;                      Calcul_dec_part				                 *
;*********************************************************************

Calcul_dec_part

	clrf	un_sur_2
	clrf	un_sur_4
	clrf	un_sur_8
	clrf	un_sur_16

	btfsc	dec,3
	call	bit3
	btfsc	dec,2
	call	bit2
	btfsc	dec,1
	call	bit1
	btfsc	dec,0
	call	bit0

	goto	affich_dec	
	
bit3
	movlw	.5
	addwf	un_sur_2,f
	return
	
bit2
	movlw	.2
	addwf	un_sur_2,f
	movlw	.5
	addwf	un_sur_4,f
	return

bit1
	movlw	.1
	addwf	un_sur_2,f	
	movlw	.2
	addwf	un_sur_4,f
	movlw	.5
	addwf	un_sur_8,f
	return

bit0
	movlw	.6
	addwf	un_sur_4,f	
	movlw	.2
	addwf	un_sur_8,f
	movlw	.5
	addwf	un_sur_16,f
	return

affich_dec

	movf	un_sur_4,w
	addlw	-.10		;W-10->W
	btfss	STATUS,C
	goto	fin_affich_dec	;un_sur_4<10
	movwf	un_sur_4
	incf	un_sur_2,f

fin_affich_dec

	return

;*********************************************************************
;                      Boucle d'attente de 10sec*W + 4 s           *
;*********************************************************************

delay_10u
		movwf	cmpt1	
delay_10u_1
		NOP	
		NOP
		NOP
		NOP	
		NOP
		NOP
		NOP
		decfsz	cmpt1,F
		goto	delay_10u_1

		return

;*********************************************************************
;                      Boucle d'attente de 1msec*W  +4s             *
;*********************************************************************

delay_1ms
		movwf	cmpt2
delay_1ms_1
		clrf	cmpt1
delay
		decfsz	cmpt1,f
		goto	delay
	
		movlw	.76
		movwf	cmpt1
delay2	
		decfsz	cmpt1,f
		goto	delay2

		decfsz	cmpt2,f
		goto	delay_1ms_1
	
		return


;*********************************************************************
;                      Boucle d'attente de 100msec*W  + 4 s             *
;*********************************************************************

delay_100ms
		movwf	cmpt3

delay_100ms_1

		movlw	.129
		movwf	cmpt2
		clrf	cmpt1
delay_100ms_2
		decfsz	cmpt1,f
		goto	delay_100ms_2
		
		nop
		nop
		nop
		nop
		nop

		decfsz	cmpt2,f
		goto	delay_100ms_2	

		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop		; 20s
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop

		decfsz	cmpt3,f
		goto	delay_100ms_1
	
		return

ERREUR_DS
		bsf		PORTC,0
		
		movlw	'N'
		call	LCD_Write_car

		movlw	'O'
		call	LCD_Write_car

		movlw	' '
		call	LCD_Write_car

		movlw	'D'
		call	LCD_Write_car

		movlw	'S'
		call	LCD_Write_car

		movlw	'.'
		call	LCD_Write_car

		movlw	.50
		call	delay_100ms

		goto	Init


	END 			; directive fin de programme
